==> Botania-1.19.2-438-FABRIC-20230214.034242-11-sources.jar: vazkii/botania/common/item/equipment/bauble/BaubleItem.java <==
/*
 * This class is distributed as part of the Botania Mod.
 * Get the Source Code in github:
 * https://github.com/Vazkii/Botania
 *
 * Botania is Open Source and distributed under the
 * Botania License: http://botaniamod.net/license.php
 */
package vazkii.botania.common.item.equipment.bauble;

import com.google.common.collect.HashMultimap;
import com.google.common.collect.Multimap;
import vazkii.botania.api.item.CosmeticAttachable;
import vazkii.botania.api.item.PhantomInkable;
import vazkii.botania.common.handler.EquipmentHandler;
import vazkii.botania.common.helper.ItemNBTHelper;
import vazkii.botania.common.helper.PlayerHelper;
import vazkii.botania.xplat.BotaniaConfig;

import java.util.List;
import java.util.UUID;
import net.minecraft.class_124;
import net.minecraft.class_1309;
import net.minecraft.class_1320;
import net.minecraft.class_1322;
import net.minecraft.class_1792;
import net.minecraft.class_1799;
import net.minecraft.class_1836;
import net.minecraft.class_1937;
import net.minecraft.class_2487;
import net.minecraft.class_2561;
import net.minecraft.class_3222;

import static vazkii.botania.common.lib.ResourceLocationHelper.prefix;

public abstract class BaubleItem extends class_1792 implements CosmeticAttachable, PhantomInkable {

	private static final String TAG_BAUBLE_UUID = "baubleUUID";
	private static final String TAG_COSMETIC_ITEM = "cosmeticItem";
	private static final String TAG_PHANTOM_INK = "phantomInk";

	public BaubleItem(class_1793 props) {
		super(props);
		EquipmentHandler.instance.onInit(this);
	}

	@Override
	public void method_7851(class_1799 stack, class_1937 world, List<class_2561> tooltip, class_1836 flags) {
		class_1799 cosmetic = getCosmeticItem(stack);
		if (!cosmetic.method_7960()) {
			tooltip.add(class_2561.method_43469("botaniamisc.hasCosmetic", cosmetic.method_7964()).method_27695(class_124.field_1080, class_124.field_1056));
		}

		if (hasPhantomInk(stack)) {
			tooltip.add(class_2561.method_43471("botaniamisc.hasPhantomInk").method_27692(class_124.field_1075));
		}
	}

	@Override
	public class_1799 getCosmeticItem(class_1799 stack) {
		class_2487 cmp = ItemNBTHelper.getCompound(stack, TAG_COSMETIC_ITEM, true);
		if (cmp == null) {
			return class_1799.field_8037;
		}
		return class_1799.method_7915(cmp);
	}

	@Override
	public void setCosmeticItem(class_1799 stack, class_1799 cosmetic) {
		class_2487 cmp = new class_2487();
		if (!cosmetic.method_7960()) {
			cmp = cosmetic.method_7953(cmp);
		}
		ItemNBTHelper.setCompound(stack, TAG_COSMETIC_ITEM, cmp);
	}

	public static UUID getBaubleUUID(class_1799 stack) {
		var tag = stack.method_7948();

		// Legacy handling
		String tagBaubleUuidMostLegacy = "baubleUUIDMost";
		String tagBaubleUuidLeastLegacy = "baubleUUIDLeast";
		if (tag.method_10545(tagBaubleUuidMostLegacy) && tag.method_10545(tagBaubleUuidLeastLegacy)) {
			UUID uuid = new UUID(tag.method_10537(tagBaubleUuidMostLegacy), tag.method_10537(tagBaubleUuidLeastLegacy));
			tag.method_25927(TAG_BAUBLE_UUID, uuid);
		}

		if (!tag.method_25928(TAG_BAUBLE_UUID)) {
			UUID uuid = UUID.randomUUID();
			tag.method_25927(TAG_BAUBLE_UUID, uuid);
		}

		return tag.method_25926(TAG_BAUBLE_UUID);
	}

	@Override
	public boolean hasPhantomInk(class_1799 stack) {
		return ItemNBTHelper.getBoolean(stack, TAG_PHANTOM_INK, false);
	}

	@Override
	public void setPhantomInk(class_1799 stack, boolean ink) {
		ItemNBTHelper.setBoolean(stack, TAG_PHANTOM_INK, ink);
	}

	public void onWornTick(class_1799 stack, class_1309 entity) {}

	public void onEquipped(class_1799 stack, class_1309 entity) {
		if (!entity.field_6002.field_9236 && entity instanceof class_3222 player) {
			PlayerHelper.grantCriterion(player, prefix("main/bauble_wear"), "code_triggered");
		}
	}

	public void onUnequipped(class_1799 stack, class_1309 entity) {}

	public boolean canEquip(class_1799 stack, class_1309 entity) {
		return true;
	}

	public Multimap<class_1320, class_1322> getEquippedAttributeModifiers(class_1799 stack) {
		return HashMultimap.create();
	}

	public boolean hasRender(class_1799 stack, class_1309 living) {
		return !hasPhantomInk(stack)
				&& BotaniaConfig.client().renderAccessories()
				&& !living.method_5767();
	}
}

==> Botania-1.20.1-451-FABRIC-20251019.112919-17-sources.jar: vazkii/botania/common/item/equipment/bauble/BaubleItem.java <==
/*
 * This class is distributed as part of the Botania Mod.
 * Get the Source Code in github:
 * https://github.com/Vazkii/Botania
 *
 * Botania is Open Source and distributed under the
 * Botania License: http://botaniamod.net/license.php
 */
package vazkii.botania.common.item.equipment.bauble;

import com.google.common.collect.HashMultimap;
import com.google.common.collect.Multimap;
import vazkii.botania.api.item.CosmeticAttachable;
import vazkii.botania.api.item.PhantomInkable;
import vazkii.botania.common.handler.EquipmentHandler;
import vazkii.botania.common.helper.ItemNBTHelper;
import vazkii.botania.common.helper.PlayerHelper;
import vazkii.botania.xplat.BotaniaConfig;

import java.util.List;
import java.util.UUID;
import net.minecraft.class_124;
import net.minecraft.class_1309;
import net.minecraft.class_1320;
import net.minecraft.class_1322;
import net.minecraft.class_1792;
import net.minecraft.class_1799;
import net.minecraft.class_1836;
import net.minecraft.class_1937;
import net.minecraft.class_2487;
import net.minecraft.class_2561;
import net.minecraft.class_3222;

import static vazkii.botania.common.lib.ResourceLocationHelper.prefix;

public abstract class BaubleItem extends class_1792 implements CosmeticAttachable, PhantomInkable {

	private static final String TAG_BAUBLE_UUID = "baubleUUID";
	private static final String TAG_COSMETIC_ITEM = "cosmeticItem";
	private static final String TAG_PHANTOM_INK = "phantomInk";

	public BaubleItem(class_1793 props) {
		super(props);
		EquipmentHandler.instance.onInit(this);
	}

	@Override
	public void method_7851(class_1799 stack, class_1937 world, List<class_2561> tooltip, class_1836 flags) {
		class_1799 cosmetic = getCosmeticItem(stack);
		if (!cosmetic.method_7960()) {
			tooltip.add(class_2561.method_43469("botaniamisc.hasCosmetic", cosmetic.method_7964()).method_27695(class_124.field_1080, class_124.field_1056));
		}

		if (hasPhantomInk(stack)) {
			tooltip.add(class_2561.method_43471("botaniamisc.hasPhantomInk").method_27692(class_124.field_1075));
		}
	}

	@Override
	public class_1799 getCosmeticItem(class_1799 stack) {
		class_2487 cmp = ItemNBTHelper.getCompound(stack, TAG_COSMETIC_ITEM, true);
		if (cmp == null) {
			return class_1799.field_8037;
		}
		return class_1799.method_7915(cmp);
	}

	@Override
	public void setCosmeticItem(class_1799 stack, class_1799 cosmetic) {
		class_2487 cmp = new class_2487();
		if (!cosmetic.method_7960()) {
			cmp = cosmetic.method_7953(cmp);
		}
		ItemNBTHelper.setCompound(stack, TAG_COSMETIC_ITEM, cmp);
	}

	public static UUID getBaubleUUID(class_1799 stack) {
		var tag = stack.method_7948();

		// Legacy handling
		String tagBaubleUuidMostLegacy = "baubleUUIDMost";
		String tagBaubleUuidLeastLegacy = "baubleUUIDLeast";
		if (tag.method_10545(tagBaubleUuidMostLegacy) && tag.method_10545(tagBaubleUuidLeastLegacy)) {
			UUID uuid = new UUID(tag.method_10537(tagBaubleUuidMostLegacy), tag.method_10537(tagBaubleUuidLeastLegacy));
			tag.method_25927(TAG_BAUBLE_UUID, uuid);
		}

		if (!tag.method_25928(TAG_BAUBLE_UUID)) {
			UUID uuid = UUID.randomUUID();
			tag.method_25927(TAG_BAUBLE_UUID, uuid);
		}

		return tag.method_25926(TAG_BAUBLE_UUID);
	}

	@Override
	public boolean hasPhantomInk(class_1799 stack) {
		return ItemNBTHelper.getBoolean(stack, TAG_PHANTOM_INK, false);
	}

	@Override
	public void setPhantomInk(class_1799 stack, boolean ink) {
		ItemNBTHelper.setBoolean(stack, TAG_PHANTOM_INK, ink);
	}

	public void onWornTick(class_1799 stack, class_1309 entity) {}

	public void onEquipped(class_1799 stack, class_1309 entity) {
		if (!entity.method_37908().field_9236 && entity instanceof class_3222 player) {
			PlayerHelper.grantCriterion(player, prefix("main/bauble_wear"), "code_triggered");
		}
	}

	public void onUnequipped(class_1799 stack, class_1309 entity) {}

	public boolean canEquip(class_1799 stack, class_1309 entity) {
		return true;
	}

	public Multimap<class_1320, class_1322> getEquippedAttributeModifiers(class_1799 stack) {
		return HashMultimap.create();
	}

	public boolean hasRender(class_1799 stack, class_1309 living) {
		return !hasPhantomInk(stack)
				&& BotaniaConfig.client().renderAccessories()
				&& !living.method_5767();
	}
}
